#include <u.h>
#include <libc.h>
#include <ip.h>
#include "../lib/snmp.h"
#include "../lib/snmp_debug.h"


void usage(char *cmd)
{
		print("Usage: %s [-D]-h <host> [-p port] [-V ver] [-r <retries>] -c <community> -o <oid> -v <value>\n", cmd);
		print("Example: %s -D -h 192.168.1.1 -p snmp -V 1 -r 2 -c public -o 1.3.6.1.2.1.1.1.0 -s s/Administrator\n", cmd);
		print("\n");
		print("Options:\n");
		print("\t-D:\t\t\t\tUse it for debug\n");
		print("\t-h <host>:\t\tSet the destination host (no default)\n");
		print("\t-p <port>:\t\tSet the destination port (default = snmp)\n");
		print("\t-V <version>:\t\tSet the SNMP version [0 for V1, 1 for 2C] (default = 1)\n");
		print("\t-r <retries>:\t\tSet the retries if something is wrong (default = 3)\n");
		print("\t-c <community>:\tSet the community (no default)\n");
		print("\t-o <OID>:\t\tSet the OID [Object ID]\n");
		print("\t-s <type>/<value>:\tThe Values's type and Value lists to Set\n");
		print("\t\tSet types:\n");
		print("\t\t\ti = integer\n");
		print("\t\t\tu = unsigned\n");
		print("\t\t\ts = string\n");
		print("\t\t\tx = hex string\n");
		print("\t\t\td = decimal string\n");
		print("\t\t\tn = nullobj\n");
		print("\t\t\to = objid\n");
		print("\t\t\tt = timeticks\n");
		print("\t\t\ta = ipaddress\n");
		print("\t\t\tb = bits\n");
		exits("Error in args");

}


int
main(int argc, char *argv[])
{
	char *cmd, *host, *port, *rocomm, *rwcomm, *val, *oids, *values, *types, **param;
	int version, retries, timeout;
	snmp_session *sess;

	/* default values */
	timeout = 5000;
	version = retries = 0;
	port = values = oids = rocomm = rwcomm = host = nil;

	param = (char**) malloc(sizeof(char*) * 100);

	cmd = argv[0];

	ARGBEGIN {
		case 'D':
			setdbg();
			break;
		case 'h':
			host=ARGF();
			break;
		case 'p':
			port=ARGF();
			break;
		case 't':
			timeout = atoi(ARGF()) * 1000;
			break;
		case 'V':
			version = atoi(ARGF());
			break;
		case 'r':
			retries = atoi(ARGF());
			break;
		case 'c':
			rwcomm = ARGF();
			break;
		case 'o':
			oids = ARGF();
			break;
		case 's':
			if ( getfields(ARGF(), param, 2, 0, "/") < 2 )
				usage(cmd);
			types = param[0];
			values = param[1];
			break;
	} ARGEND;

	if (host == nil)
		usage(cmd);

	sess = mksession(version, host, port, rocomm, rwcomm, timeout, retries);

	if (sess ==nil) {
		print("Error creating snmp session.\n");
		exits("Error creating session\n");
		return(1);
	} else {

		val = snmpCmd(sess, oids, values, types, 0, 0, SetRequest);
		
		if (val != nil) {
			print("%s", val);
			free(val);
		}

		freesession(sess);
		/* while(1) { }  This code is only for debug, and leaks */
		return 0;		
	}
}
