#include <u.h>
#include <libc.h>
#include <bio.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include "../lib/snmp_debug.h"
#include "../lib/snmp_session.h"


snmp_session *
mksession(int version, char *host, char *port, char *rocommunity, char *rwcommunity, int tmout, int retrs)
{
	snmp_session *sess;

	snmpdbg(DEBUG, PLUS, "mkssesion()", "Creating a SNMP session.");

	/* Destination Host */
	if (host == nil) {
		snmpdbg(DEBUG, MINUS, "mkssesion()", "No host, no session.");
		return nil;
	}

	/* memory!! */
	sess = emalloc9p(sizeof(snmp_session));

	if (sess == nil) {
		snmpdbg(DEBUG, MINUS, "mkssesion()", "No memory for the session.");
		return nil;
	}

	/* Value's Asignation */
	/* ****************** */

	/* Host */
	sess->host = strdup(host);

	/* Version */
	if (( version < 0 ) || ( version > 3 )) {
		version = SNMP_VERSION;
	}
	sess->version = version;

	/* Destination Port */
	if (port == nil) {
		sess->port = smprint("%s", SNMP_PORT);
	} else {
		sess->port = strdup(port);
	}

	/* RO Community */
	if (rocommunity == nil) {
		sess->rocommunity = smprint("%s", SNMP_ROCOMMUNITY);
	} else {
		sess->rocommunity = strdup(rocommunity);
	}
	sess->rocommlen = strlen(sess->rocommunity);

	/* RW Community */
	if (rwcommunity == nil) {
		sess->rwcommunity = smprint("%s", SNMP_RWCOMMUNITY);
	} else {
		sess->rwcommunity = strdup(rwcommunity);
	}
	sess->rwcommlen = strlen(sess->rwcommunity);

	/* Timeout */
	if (tmout == 0) {
		sess->timeout = SNMP_TIMEOUT;
	} else {
		sess->timeout = tmout;
	}

	/* Retries */
	if (retrs == 0) {
		sess->retries = SNMP_RETRIES;
	} else {
		sess->retries = retrs;
	}

	/* Values sets in the code, kix change it to parameter */
	sess->sndMsgMaxSize = SNMP_MAX_SND_PKT;
	sess->rcvMsgMaxSize = SNMP_MAX_RCV_PKT;
	sess->maxoids = SNMP_MAX_OIDS;

	snmpdbg(DEBUG, MINUS, "mkssesion()", "Session created.");

	return sess;
}


void
freesession(snmp_session *sess)
{
	snmpdbg(DEBUG, PLUS, "freesession()", "Freeing the memory of Session.");

	free(sess->host);
	free(sess->port);
	free(sess->rocommunity);
	free(sess->rwcommunity);
	free(sess);

	snmpdbg(DEBUG, MINUS, "freesession()", "Memory freed.");
}
